# coding: utf-8
"""
mixin classes from which analysis tasks can inherit
"""
import os
import luigi

from functools import partial

from analysis.util import log_maybe_raise


class TarballExtractionMixin:
    def extract_tarball(self, src_target, dst_target, formatter="tar", logger=None, check_targets=None):
        """
        Extract the contents of an archive pointed to by *src_target* to a local
        directory, indicated by *dst_target*. Both arguments should be ``law.Target``
        objects, and *dst_target* should point to a directory.

        Optionally, a list of *check_targets* can be provided. These will be checked to exist
        after the tarball extraction, and an exception will be raised otherwise.
        """

        # validate formatter
        if formatter not in ("tar", "gzip"):
            raise ValueError(
                "formatter '{}' not supported; use one of: tar, gzip".format(
                    formatter,
                )
            )

        # logging helper
        _log = partial(log_maybe_raise, logger=logger)

        # report on input and output targets
        _log("debug", "src_target is %s", src_target)
        _log("debug", "dst_target is %s", dst_target)

        # throw error if source file doesn't exist
        if not src_target.exists():
            _log(
                "error",
                "copy source not found: %s",
                src_target,
                raise_cls=RuntimeError,
            )

        # ensure output directory exists
        dst_target.touch()

        # extract tarball to parent directory
        _log(
            "info",
            "copying result: %s -> %s",
            src_target,
            dst_target,
        )
        src_target.load(dst_target, formatter=formatter)

        # check expected files after extraction
        missing = [
            t
            for t in (check_targets or [])
            if not t.exists()
        ]

        # return if no missing files
        if not missing:
            return

        # inform about missing files and raise exception
        message = (
            "%s (branch %s) did not copy expected file(s) from %s"
        )
        message += "\nExpected files:" + "".join(
            "\n  - {}".format(target.basename)
            for target in missing
        )
        dir_contents = os.listdir(dst_target.path)
        if dir_contents:
            message += "\nDirectory contents:" + "".join(
                "\n  - {}".format(fname)
                for fname in dir_contents
            )
        _log(
            "error",
            message,
            self.__class__.__name__,
            self.branch,
            src_target,
            raise_cls=RuntimeError,
        )


class HTCondorRemoteStorageMixin:
    # Set default for new luigi parameter htcondor_remote_storage
    htcondor_remote_storage = luigi.BoolParameter(default=True)

    def remote_or_local_target(self, output_filename, logger=None):
        """
        Return a local or a remote file target, depending on
        the value of the `htcondor_remote_storage` parameter.
        """
        _log = partial(log_maybe_raise, logger=logger)

        # if remote storage is enabled, make remote target and return it
        if self.htcondor_remote_storage:
            target = self.remote_target(output_filename)
            _log("debug", "Remote output target is: %s", target)
            return target
        else:
            target = self.local_target(output_filename)
            _log("debug", "Local output target is: %s", target)
            return target
